=begin rdoc
Base definitions of rake4latex.
This file must be loaded by all specific rake4file-variants.
=end

#~ gem 'rake', '=0.8.7'
require 'rake'
case RUBY_VERSION
  when /1.8/;   
    require 'md5'             #   defines MD5 and  Digest::MD5    
  when /1.9/, /^2./   
    require 'digest/md5'  #defines Digest::MD5
    require 'yaml'
  else
    raise "Undefined ruby version #{RUBY_VERSION}"
end
  
require 'log4r' #1.0.5
#Some Tools have screen output.
#Rake4LaTeX should run silent in background, so we have to redirect the output.
require "open3"
=begin
      stdin, stdout, stderr, wait_thr = Open3.popen3(@cmd)
      @stdout = stdout.readlines.join
      @stderr = stderr.readlines.join
      if wait_thr.value.exitstatus != 0

  stdin, stdout, stderr, wait_thr = Open3.popen3(cmd)
  if wait_thr.value.exitstatus != 0
    task.texrunner.logger.fatal("...\n#{stdout.readlines.join}")
  end

=end


module Rake
  class Task
=begin rdoc
Define and get the related TeXRunner.
The value is set by Rake4LaTeX::LaTeXRunner#run_latex_once
and can be used by the tex_postrule.
=end
    attr_accessor :texrunner
  end
end

module Rake4LaTeX
  VERSION = '0.1.5'
  #Collect descriptions for tasks. See #mk_taskdescriptions
  TASK_DESCRIPTION = true
  TASK_DESCRIPTIONS = {}

  #With this creation we define the constants Log4r::DEBUG...
  Logger = Log4r::Logger.new("LaTeXRunner")
  Logger.outputters = Log4r::StdoutOutputter.new('log_stdout')

  #
  #Define the main settings for the runner.
  #
  #This values can be overwritten for each base file:
  #   basefile = Rake4LaTeX::Basefile.set('filename')
  #   basefile[:maxruns] =  1
  #   basefile[:loglevel] =  Log4r::DEBUG
  #   basefile[:program] =  :xelatex
  #   basefile[:texerrors_allowed] =  true
  #
  DEFAULT_SETTINGS = {
    :main_file => nil,  #only needed for key-check. Must be defined later.
    :maxruns => 5,
    :program => :pdflatex,  #Valid entries see PROGRAMS
    :loglevel => Log4r::INFO,
    :texerrors_allowed => false,
  }

  #Adaption per file:
  # Rake4LaTeX::Basefile.set(basename)[:loglevel] =  Log4r::DEBUG
  Logger.level = DEFAULT_SETTINGS[:loglevel]
  
=begin rdoc
Definition of the programms and there call parameters.

There is a basic definition, you may redefine the values with your 
local definitions.
You can define your local definition in a file 'rake4latex.yaml'.

Each program is a Hash:
  program:
    cmd: shell-cmd
    parameters:
      value: value as textstring
      name: name of the parameter
      value_key: key of the value, must be given to Rake4LaTeX.build_cmd
      optional: flag, if the parameter is optional.
      space_separated: between name and value is a space.
=end
Programms = {}

=begin rdoc
Load the Programs with parameters for Rake4LaTeX.build_cmd.
=end
  [
    "#{File.dirname(__FILE__)}/rake4latex.yaml",
    #ocra-settings:
    #a src\lib\rake4latex\rake4latex.yaml    
    #a lib\ruby\gems\1.8\gems\rake4latex-0.1.0\lib\rake4latex\base.rb    
    "#{File.dirname(__FILE__)}/../../../../../../../../src/lib/rake4latex/rake4latex.yaml",  #for ocra-version (exe)
    #fixme: user/local system specific settings
    "./rake4latex.yaml", #local re-definition
  ].each{| setting_file |
    if File.exist?(setting_file)
      #Take the "more local" definition.
      Programms.replace(YAML.load(File.read(setting_file)).merge(Programms))
    end
  }
  if ! defined? Programms
    raise "No program settings found"
  end

=begin rdoc
Define all descriptions for tasks.

This defines the descriptions:
  rake basefile           # Set the basefile - needed for clean and statistic
  rake clean[basename]    # Remove any temporary products
  rake clobber            # Remove any generated file
  rake log_archive        # Archive all log-files to zip-file
  rake log_overview       # Show the log-overview
  rake log_overview_file  # Build a log-overview file
  rake pdfview            # Show compiled PDF in Reader
  rake statistic          # Build a statistic for all TeX project
  rake touch              # Touch - reset timestamp
=end
  def self.mk_taskdescriptions
    TASK_DESCRIPTIONS.each{|taskname, descr|
      Rake.application[taskname].comment = descr
    }
  end
=begin rdoc
Build the command line to call a tool.

Bases is the configuration in Programms.

'parameters' contains the parameters for the call.
The keys must fit to the settings in Rake4LaTeX::Programms .
=end
  def self.build_cmd( programm, parameters = {}, task = nil )
    
    if task #take the LaTeXRunner specific settings
      configuration = task.texrunner.programms[programm] 
    else  #take the global settings
      configuration = Programms[programm] 
    end
    if ! configuration
      Rake4LaTeX::Logger.fatal( "No configuration for #{programm.inspect}") if Rake4LaTeX::Logger.fatal?
      return false
    end
    
    cmd = configuration['cmd'].dup
    configuration['parameters'].each{ | parameter |
      cmd << " #{parameter['value']}" if parameter['value']
      if parameter['value_key']
        if parameters[parameter['value_key']]
          cmd << " "
          cmd << "#{parameter['name']}" if parameter['name']
          cmd << " " if parameter['space_separated']
          cmd << "#{parameters[parameter['value_key']]}"
        else
          if ! parameter['optional'] and Rake4LaTeX::Logger.error?
            Rake4LaTeX::Logger.error( "Parameter #{parameter['value_key'].inspect} missing to execute #{programm}") 
          end
        end
      end
    }#configuration['parameters']
  
    cmd
  end
=begin rdoc
Define a basename.

Set the file for clean/clobber, offers the possibility to set defaults,
define a logger.
=end
  class Basefile
    #needed for Rake-version  >= 0.9.1
    include Rake::DSL if defined? Rake::DSL
    @@all = {}
    
=begin rdoc    
Get the basefile object or create a a new instance.
=end
    def self.set( basename, options = {} )
      basename = basename.ext()
      @@all[basename] = new(basename) unless @@all[basename]
      options.each{|key, value| @@all[basename][key] = value }
      @@all[basename]
    end #set
    
=begin rdoc    
Get the basefile object.
=end
    def self.[](key)
      @@all[key.ext()]
    end
=begin rdoc
Loop on all defined base files
=end
    def self.each()
      @@all.each{|key, basefile| yield basefile }
    end

=begin rdoc
Create a new basefile object.

This is a private method.
To create new objects, use Rake4LaTeX::Basefile.set
=end
    def initialize( basename, options = {} )
      @basename = basename.ext('')
      
      @settings4LaTeXRunner = DEFAULT_SETTINGS.dup
      
      @logger = Log4r::Logger.new(@basename)
      @logger.level = @settings4LaTeXRunner[:loglevel]

      options.each{|key, value| self[key] = value }

      @logger.outputters = Log4r::StdoutOutputter.new('log_stdout')
      #~ @logger.info("Define base file #{@basename}")
      
      
      set4clean()
      define_tasks()    
    end #basename#initialze
    private_class_method :new #make it private

    #Basename of the Basefile.
    attr_reader :basename
    #Get the logger
    attr_reader :logger
    #Settings for the LaTeXRunner. May be changed with Rake4LaTeX::Basefile#[]=
    attr_reader :settings4LaTeXRunner
=begin rdoc
Defines document specific tasks (statistic, set clean and clobber)
=end
    def define_tasks()    
      
      namespace @basename do
        desc "Create the statistic for #{@basename}" if Rake4LaTeX::TASK_DESCRIPTION 
        task :statistic do
          stat = Rake4LaTeX::TeX_Statistic.new(@basename)
          puts "Statistic for #{@basename}:"
          puts stat.stat_summary.map{|e| "  #{e}"}
        end

        desc "Build a log-overview for #{@basename}" if Rake4LaTeX::TASK_DESCRIPTION 
        task :log_overview do
            stat = Rake4LaTeX::TeX_Statistic.new(@basename)
            puts stat.overview()
        end

        desc "Build a log-overview file for #{@basename}" if Rake4LaTeX::TASK_DESCRIPTION 
        task :log_overview_file do
            stat = Rake4LaTeX::TeX_Statistic.new(@basename)
            #Filename also used in set4clean
            File.open("#{@basename}.overview.txt", 'w'){|f| 
              puts "Create #{f.path}"
              f << stat.overview()
            }
        end

      end  #namespace
      
      #Define the 'global' tasks.
      task :statistic       => "#{@basename}:statistic"
      task :log_overview => "#{@basename}:log_overview"
      task :log_overview_file => "#{@basename}:log_overview_file"
    end #define_tasks()


=begin rdoc
Define the files to be deleted with clean and clobber.

There is no global definition to delete all files, 
only the help file for the selected basename is taken.

This method is called by task 'basename' (Basefile#initialize)
=end
    def set4clean()
      cleanlist = FileList.new("#{@basename}.{aux,log,out,toc,lot,lof,nav,snm}")
      cleanlist.add("#{@basename}.{ilg,idx,ind}") #Index
      cleanlist.add("#{@basename}-*.{ilg,idx,ind}") #splitindex.sty
      cleanlist.add("#{@basename}.{blg,bbl}") #BibTeX
      cleanlist.add("#{@basename}.{bcf}") #BibLaTeX
      cleanlist.add("#{@basename}-blx.{bib}") #auxiliary file used by the 'biblatex' package
      cleanlist.add("#{@basename}.*.{blg,bbl,aux}") #gloss.sty
      cleanlist.add("#{@basename}.{maf,ptc*,mtc*,stc*}") #minitoc.sty (stc1, stc2...)
      cleanlist.add("#{@basename}.{glo,gls}") #glossary.sty
      #~ cleanlist.add("#{@basename}.{rai,rao,RAO}") #rail/rail.sty
      cleanlist.add("#{@basename}.{rai,rao}") #rail/rail.sty
      cleanlist.add("#{@basename}.{tdo}") #tdo: todonotes.sty
      cleanlist.add("#{@basename}.{vrb}") #vrb: beamer.sty (verbatim)
      cleanlist.add("#{@basename}.{lox}") #lox: fixme.sty 
      cleanlist.add("#{@basename}.{run.xml}") #run.xml: TeXworks (editor)
      cleanlist.add("#{@basename}.{upa,upb}") #pdfcomment.sty

      clobberlist = FileList.new("#{@basename}.{dvi,pdf,ps}")
      #The * is needed.else the filename is added always. With '*' it is checked for existence
      clobberlist.add("#{@basename}.overview*.txt")  #rake4latex, task :log_overview
      
      CLEAN.include(cleanlist)
      CLOBBER.include(clobberlist)

      CLEAN.uniq!
      CLOBBER.uniq!
      
    end #set4clean

=begin rdoc
Sets a default option for the LaTeXRunner.

The options are stored and used for new LaTeXRunners.

Examples:
  
  basefile = Rake4LaTeX::Basefile.set('filename')
  basefile[:maxruns] =  1
  basefile[:loglevel] =  Log4r::DEBUG
  basefile[:program] =  :xelatex
  basefile[:texerrors_allowed] =  true

=end
  def []=( key, option )
    if ! settings4LaTeXRunner.keys.include?(key)
      raise ArgumentError, "Undefined key #{key.inspect} for LaTeXRunner" unless @latexrunner
      @ogger.warn("Undefined key #{key.inspect} for LaTeXRunner") if @ogger.warn?
    end
    settings4LaTeXRunner[key] = option
    case key
      when :program
      if LaTeXRunner::PROGRAMS.keys.include?( option.to_sym )
        settings4LaTeXRunner[key] = option.to_sym
      else
        @ogger.warn("Undefined programm #{option.inspect} for LaTeXRunner") if @logger.warn?
      end
    end
  end


    def inspect()
      "#<Rake4LaTeX::Basefile #{@basename}>"
    end
  end #Basefile
end #Rake4LaTeX


=begin rdoc
Define a task to be executed after each TeX run.

The task name is added to LaTeXRunner::Post_Prerequisites.
LaTeXRunner#run_latex_once will loop on all tasks in LaTeXRunner::Post_Prerequisites.

In the task definition you have access to the TeXRunner via Rake::Task#texrunner .
=end
def tex_postrule(*args, &block)
  #~ rules = rule(*args, &block)
  Rake::Task.create_rule(*args, &block)  #@rules << [pattern, deps, block]
  #
  if args.size == 1 #normal rule without arguments
    Rake4LaTeX::LaTeXRunner::Post_Prerequisites << args.first.keys.first
  else  #rule with arguments (args.last is a hash with :needs)
    Rake4LaTeX::LaTeXRunner::Post_Prerequisites << args.first
  end
end
=begin rdoc
Define a procedure to decide, if the post process should be 
called.

The task name is added to LaTeXRunner::Post_Prerequisites_check.
LaTeXRunner#run_latex_once will loop on all tasks in LaTeXRunner::Post_Prerequisites_check.

The block will be called with a hash, the block must accept this one parameter.

Inside the block you have access to:
* :task: The task for which you test.
* :checksums: the checksums of the files before the TeX-run.
* :texrunner: The actual TeXRunner, calling the pre-check.
* :logger: the logger of the related task

The block must return false or the reason for a call (e.g. "testdocument.aux changed")

Example: See BibTeX-definition.
=end
def tex_postrule_check(rulename, &block)
  raise "No block for postrule_check #{rulename}" unless block_given?
  raise "Wrong number of arguments for postrule_check #{rulename}" unless block.arity == 1
  Rake4LaTeX::LaTeXRunner::Post_Prerequisites_check[rulename] ||= [] << block
end

#
#Load the sub-files
#
require_relative 'clean'  #modified rake/clean
require_relative 'latexrunner'
require_relative 'analyse_texfile'
require_relative 'splitindex'
require_relative 'rail'
require_relative 'rules'
require_relative 'tex_statistic'

#~ Rake4LaTeX.mk_taskdescriptions if Rake4LaTeX::TASK_DESCRIPTION 
