#!/usr/bin/env ruby

# The 'rake/clean' file defines two file lists (CLEAN and CLOBBER) and
# two rake tasks (:clean and :clobber).
#
# [:clean] Clean up the project by deleting scratch files and backup
#          files.  Add files to the CLEAN file list to have the :clean
#          target handle them.
#
# [:clobber] Clobber all generated and non-source files in a project.
#            The task depends on :clean, so all the clean files will
#            be deleted as well as files in the CLOBBER file list.
#            The intent of this task is to return a project to its
#            pristine, just unpacked state.
#
# This replaces the original clean.
#
# Changes:
# * Use File.delete (avoid stderr-messages of rm_r)
# * Use LaTeXRunner.logger if available.
# * Use uniq to avoid warnings for just deleted files.
# * Add a parameter basefile
#

require 'rake'

CLEAN = Rake::FileList["**/*~", "**/*.bak", "**/core"]
CLEAN.clear_exclude.exclude { |fn| 
  fn.pathmap("%f") == 'core' && File.directory?(fn) 
}

Rake4LaTeX::TASK_DESCRIPTIONS[:clean] = "Remove any temporary products. Optional parameter: Basefile"
#~ task :clean do
task :clean, :basename do |t, args|
  Rake4LaTeX::Basefile.each{|file| file.set4clean } #late call is necessary. doc. says FileList would work without...
  if args[:basename]
    Rake4LaTeX::Basefile.set(args[:basename]).set4clean  #add given parameter to be deleted.
  end
  
  logger = Rake4LaTeX::Logger  
  logger.info("Clean: delete helpfiles #{CLEAN.inspect}") if logger.info?
  CLEAN.uniq.each { |fn| 
    #~ rm_r fn rescue nil 
    #~ logger.debug("Clean: delete #{fn}") if logger
    File.delete(fn) rescue ( logger.warn("Clean: #{fn} not deleted") if logger )
    CLEAN.delete(fn)  #avoid warning if clean is called twice.
  }
end

CLOBBER = Rake::FileList.new

Rake4LaTeX::TASK_DESCRIPTIONS[:clobber] = "Remove any generated file."
task :clobber => :clean do
  logger = Rake4LaTeX::Logger
  logger.info("Clobber: delete helpfiles #{(CLEAN + CLOBBER).inspect}") if logger.info?
  CLOBBER.uniq.each { |fn| 
    #~ rm_r fn rescue nil 
    #~ logger.debug("Clobber: delete #{fn}") if logger
    File.delete(fn) rescue ( logger.warn("Clobber: #{fn} not deleted") if logger )
  }
end
