#encoding: utf-8
=begin rdoc
Define tasks for usage of rake4latex with glossaries.sty

More about glossaries.sty:
* http://www.ctan.org/tex-archive/help/Catalogue/entries/glossaries.html
* http://www.ctan.org/tex-archive/macros/latex/contrib/glossaries/
* http://theoval.cmp.uea.ac.uk/~nlct/latex/packages/glossaries/glossaries-manual.html

ToDo:
The logs from the glossaries are not analysed by Rake4LaTeX::TeXMessage
=end


=begin
AUX-File:

\@newglossary{label}{log}{out-ext}{in-ext}

\@newglossary{main}{glg}{gls}{glo}
\@newglossary{ext}{TeX-extensions}{glsext_in}{glsext_out}
\@istfilename{sample.ist}

Anschliessend makeindex rufen.

Statt ind/idx werden 
- gls => glo (default)
- glsext_in => glsext_out (selbstdefiniert)
verwendet.

Zusätzlich Index-file ist.
=end


module Rake4LaTeX
=begin
Extend the class by accessors for glossaries.sty.
=end
  class LaTeXRunner
=begin rdoc
Store information from glossaries.sty to transport it from
tex_postrule_check to tex_postrule for 'gls'-task.
=end
  attr_accessor :glossaries
=begin rdoc
Format for the makefindex call neeeded for glossaries.sty.
=end
  attr_accessor :glossaries_format
  end #LaTeXRunner
end #Rake4LaTeX

=begin
Check glossaries.sty-infos after a TeX-run.

The data are stored in Rake4LaTeX::LaTeXRunner#glossaries
and Rake4LaTeX::LaTeXRunner#glossaries_format .
=end
desc "Build glossary with glossaries.sty"
tex_postrule_check '.gls' do |args|
  gls = {}
  File.read(args[:task].name.ext('aux')).each_line{|auxline|
    case auxline
      when /\\@newglossary\{(.*)\}\{(.*)\}\{(.*)\}\{(.*)\}/
        args[:logger].debug("Found Glossary #{$1}-#{$2}-#{$3}-#{$4}")
        gls[$1] = { :log => $2, :out => $3, :in => $4 }
      when /\\@istfilename\{(.*)\}/
        args[:logger].debug("Found Glossary format definition #{$1}")
        args[:texrunner].glossaries_format = $1
    end
  }

  #Provide glossaries-data to gls-task.
  args[:texrunner].glossaries = gls
  gls.empty? ? false : true
end


=begin
Build the glossaries from glossaries.sty.

The data are taken from Rake4LaTeX::LaTeXRunner#glossaries
and Rake4LaTeX::LaTeXRunner#glossaries_format .
=end
desc "Call Makeindex for glossaries with glossaries.sty"
tex_postrule '.gls' => '.glo' do | t |
  #fixme: check for t.texrunner.glossaries
  t.texrunner.logger.info("Prepare glossaries #{t.texrunner.glossaries.keys.inspect}")
  t.texrunner.logger.warn("No makeindex-style available for glossaries") unless t.texrunner.glossaries_format
  t.texrunner.glossaries.each{|key, par|
    cmd = Rake4LaTeX.build_cmd( 'makeindex', 
          :file_in => t.name.ext(par[:in]), :file_out => t.name.ext(par[:out]),
          :file_log => t.name.ext(par[:log]), :format => t.texrunner.glossaries_format
            )
#~ puts cmd
    Rake4LaTeX::Logger.debug("Call #{cmd}")
    stdin, stdout, stderr, wait_thr = Open3.popen3(cmd)
    CLEAN.include(FileList.new("#{t.name.ext()}.{#{par[:in]},#{par[:out]},#{par[:log]}}"))
  }#$gls.each
  CLEAN.include(FileList.new(t.texrunner.glossaries_format))  if t.texrunner.glossaries_format
end
