module Rake4LaTeX
=begin rdoc
Support rail-diagramms in LaTeX.

Details for the rail package see http://www.ctan.org/tex-archive/support/rail/

=end
  class Rail
    def initialize(source, logger = Log4r::Logger.new('log'))
      @source = source
      @log = logger
      @cmd = Rake4LaTeX.build_cmd( 'rail', :railfile => source )
    end
    #Command line to start rail
    attr_reader :cmd
    attr_reader :stdout
    attr_reader :stderr
=begin rdoc
Execute rail and returns the result.

rail.exe does not support filenames longer with 8 characters.
So we use stdin:

  rail < input.rai

Returns successcode (true/false)

=end
    def execute
      #execute
      @log.debug("Call rail: #{@cmd}")
      stdin, stdout, stderr, wait_thr = Open3.popen3(@cmd)
      
      #analyse result
      success = true
      #check return code 
      if wait_thr.value.exitstatus != 0
        @log.fatal("There where Rail errors.")
        success = false
      end
      @stdout = stdout.readlines.join
      @stderr = stderr.readlines.join
      #check errors
      @stderr.each_line{|errline|
        @log.error("Rail error: #{errline.strip}")
        success = false
      }
      
      if ! success
        @log.warn("No rail diagramms generated")
      end

      #Now we have to save the result and delete some stuff.
      if @stdout
        @stdout.sub!(/^(This is Rail.*)/, '') #%This is Rail, Version 1.1 #0
        @stdout.sub!(/^(\(stdin.*)/, '')  
        @stdout.sub!(/^\)/, '')		#last line
        @stdout.strip!
      end
      
      success
    end #execute
  end #class Rail
end #module Rake4LaTeX


if $0 == __FILE__
    require_relative '../rake4latex'
    @@testrai_file = '_tmp_.rai'
    File.open(@@testrai_file, 'w'){|f|
      f << "\\rail@i{1}{ block1 : ( lbrace | 'begin' ) ( ( '|' ( variable + ',' ) '|' ) | [Keine Blockvariablen] ) comm ( rbrace | 'end' ) ; }"
      f << "\\rail@i{1}{ block1 : ( lbrace | "
    }
    @rail = Rake4LaTeX::Rail.new(@@testrai_file)
    p @rail.cmd
    p @rail.execute
    p @rail.stdout
    p @rail.stderr
    
end