#encoding: utf-8
=begin rdoc
Collection of rules to build a LaTeX-project.

This file defines how to build:
* dvi (latex)
* ind (makeindex)
* bbl (BibTeX)
* ps (ps2pdf)

There is no definition how to create a pdf.
This may be done via different ways. You can select your version on your own:
* ../rake4latex_dvipdfm.rb
* ../rake4latex_lualatex.rb
* ../rake4latex_pdflatex.rb (Default)
* ../rake4latex_ps2pdf.rb
* ../rake4latex_xelatex.rb

=end

#
#Generate pdf-rules/files
#
#~ %w{xe lua}.each{|x|
  #~ File.open("../rake4latex_#{x}latex.rb", 'w'){|f| f << File.read('../rake4latex_pdflatex.rb').gsub(/pdf(latex)/i, "#{x}\\1")}
#~ }
#~ Dir['../rake4latex_*'].each{|x| puts "* #{x}" }


Rake4LaTeX::TASK_DESCRIPTIONS[:pdfview] =  "Show compiled PDF in Reader."
task :pdfview do |t|
  t.prerequisites.each{|pre|
    #Problem: rake process waits until reader is closed
    sh "call #{pre.ext('pdf')}"
  }
end

=begin
Alternative definition instead Rake4LaTeX::Basefile.set(filename)
=end
Rake4LaTeX::TASK_DESCRIPTIONS[:basefile] =  "Set the basefile - needed for clean and statistic"
task :basefile do |task|
  task.prerequisites.each{|pre|
    #~ puts "Set basefile #{pre.inspect}"
    puts "Task basefile defined for #{pre.inspect} (Please replace it with: Rake4LaTeX::Basefile.set)"
    Rake4LaTeX::Basefile.new(pre)
  }
end

#
#Reset the timestamp of a file (can be used to force the first compilation)
#
Rake4LaTeX::TASK_DESCRIPTIONS[:touch] =  "Touch - reset timestamp"
task :touch do |task|
  task.prerequisites.each{|pre|
    FileUtils.touch(pre)
  }
end

=begin
Call LaTeX
=end
desc "Build a dvi-file with LaTeX"
rule '.dvi' => '.tex' do |task|
  runner =   Rake4LaTeX::LaTeXRunner.new( 
    :main_file => task.source,
    :program => :latex,
    :dummy => nil
    )
  runner.execute  #Does all the work and calls the "post-prerequisites"
end

=begin
Call dvips
=end
desc "Build a ps-file from dvi"
rule '.ps' => '.dvi' do |task|
  
  Rake4LaTeX::Basefile.set(task.source).logger.info("Call dvips for <#{task.source}>")
  cmd = Rake4LaTeX.build_cmd( 'dvips', :filename => task.source )
  
  stdin, stdout, stderr, wait_thr = Open3.popen3(cmd)
  if wait_thr.value.exitstatus != 0
    task.texrunner.logger.fatal("There where dvips errors. \n#{stdout.readlines.join}")
  end
end

=begin
Call Makenindex
=end
desc "Call Makeindex"
tex_postrule '.ind' => '.idx' do |task|
  #check for splitidx
  #If you use index.sty you will get an error.
  splitidx = Rake4LaTeX::Splitindex.new(task.source, task.texrunner.logger)
  if splitidx.makeindex?   #standard index, no splitidx
    #makeindex writes to stderr -> catch it
    cmd = Rake4LaTeX.build_cmd( 'makeindex', { :file_in => task.source }, task )
    task.texrunner.logger.debug("\t#{cmd}")
    stdin, stdout, stderr, wait_thr = Open3.popen3(cmd)
    if wait_thr.value.exitstatus != 0
      @logger.fatal("There where Makeindex errors.\n#{stderr.readlines.join}")
    end
  else  #splitidx used
    #call splitindex
    splitidx.execute()
  end
end

=begin
Call BibTeX
=end
desc "Call BibTeX"
#Define the precheck
tex_postrule_check '.bbl' do |args|
  necessary = false
  #Check for change in aux-file
  #This must be done outside the prerequisites-loop.
  #With definition
  #       file 'testdocument.pdf' => 'testdocument.bib'
  #the auxfile is missing in prerequisites and we loose the 3rd run.
  auxfile = args[:task].name.ext('aux')
  if File.exist?(auxfile)
    #BibTeX should only start when BibTeX is really used.
    if ( args[:checksums][auxfile] == :changed and File.read(auxfile) =~ /bibdata/ )
      necessary = "#{auxfile} changed" 
    end
  else
    args[:logger].warn("\t#{auxfile} not found in bbl-precheck")
  end
  
  #Check, if the bbl is missing.
  #may occur, if the bbl-file was deleted manual, but no change in aux...
  # fixme -- provoziert unnoetige bibtex-laeufe.
  #~ bblfile = args[:task].name.ext('bbl')
  #~ necessary = "#{bblfile} missing"  unless File.exist?(bblfile)
    
  #Check all prerequisites.
  #This should be the aux-file and (optional) bib-files.
  args[:task].prerequisites.each{|post_prereq_pre|  
    args[:logger].debug("\tCheck dependency <#{post_prereq_pre}>")
    case post_prereq_pre
      when /\.aux\Z/  #Already done
      when String #Name of the file (normally xxx.bib)
        #compare timestamps of last modification
        if File.exist?(post_prereq_pre) 
          if args[:task].timestamp < File.mtime(post_prereq_pre)
            args[:logger].debug("Bibbliography <#{post_prereq_pre}> changed")
            necessary = "#{post_prereq_pre} changed"
          end
        else  #File not found
          args[:logger].warn("Bibbliography: File <#{post_prereq_pre}> not found for pre-check")
          necessary = "#{post_prereq_pre}"
        end #File.exist?
      else
          args[:logger].warn("Bibbliography: Don't know how to pre-check <#{post_prereq_pre.inspect}>")
          necessary = "#{post_prereq_pre}"
      end #case post_prereq_pre
      break if necessary
  } unless necessary
  necessary #return parameter
end

#Define the rule
tex_postrule '.bbl' => '.aux' do |task|  
  cmd = Rake4LaTeX.build_cmd( 'bibtex', {:source => task.source}, task )
  task.texrunner.logger.debug("Call bibTeX: #{cmd}")
  stdin, stdout, stderr, wait_thr = Open3.popen3(cmd)
  if wait_thr.value.exitstatus != 0
    task.texrunner.logger.fatal("There where BibTeX errors. \n#{stdout.readlines.join}")
  end
end

=begin
Call Rail
Details see Rake4LaTeX::Rail
=end
desc "Call Rail"
tex_postrule '.rao' => '.rai' do |task|
  rail = Rake4LaTeX::Rail.new(task.source, task.texrunner.logger)
  if rail.execute
    File.open(task.source.ext('rao'), 'w'){|f| f << rail.stdout }  
  end
  CLEAN.include(task.source)
  CLEAN.include(task.name)
end

=begin
End Task and TeX-independend tasks.

This tasks require a previous Rake4LaTeX::Basefile.set
=end



=begin
Build Statistic.

The main action is defined in Rake4LaTeX::Basefile
=end
desc nil  #reset previous desc definitions
Rake4LaTeX::TASK_DESCRIPTIONS[:statistic] = "Build a statistic for all TeX project"
task :statistic 

=begin
Show the log-overview
=end
Rake4LaTeX::TASK_DESCRIPTIONS[:log_overview] =  "Show the log-overview"
task :log_overview

=begin
Build a log-overview file
=end
Rake4LaTeX::TASK_DESCRIPTIONS[:log_overview_file] =  "Build a log-overview file"
task :log_overview_file

=begin
Archive all log-files to zip-file
=end
Rake4LaTeX::TASK_DESCRIPTIONS[:log_archive] =  "Archive all log-files to zip-file"
task :log_archive do
  require 'zip/zipfilesystem'
  Zip::ZipFile.open("rake4latex_log_archive.zip", Zip::ZipFile::CREATE ){|zip|
    puts "Pack #{zip.name}"
    #Store all Files to be deleted
    CLEAN.each{|filename|
      zip.file.open(File.basename(filename), 'w'){|f| f << File.open(filename,"rb") {|io| io.read} }
    }
    Rake4LaTeX::Basefile.each{|basefile|
      stat = Rake4LaTeX::TeX_Statistic.new(basefile.basename)
      zip.file.open("#{basefile.basename}.overview.txt", 'w'){|f| 
        f << stat.overview()
      }
    }    
  } #close zip-file
end